;; mudlle code editing mode for Emacs: version 1.01
;; Modified c-mode.el by David Gay (dgay@di.epfl.ch) and
;; Mattias Engdegard (f91-men@nada.kth.se) 1993.
;; c-mode.el is Copyright (C) 1985, 1986, 1987 Free Software Foundation, Inc.

;; This file is based on code which is part of GNU Emacs. It is therefore
;; subject to the same General Public License as GNU Emacs. See the COPYING
;; file for details. Note, that this license does NOT apply to any part of
;; the Cancan client, with which it is distributed.

;; GNU Emacs is free software; you can redistribute it and/or modify
;; it under the terms of the GNU General Public License as published by
;; the Free Software Foundation; either version 1, or (at your option)
;; any later version.

;; GNU Emacs is distributed in the hope that it will be useful,
;; but WITHOUT ANY WARRANTY; without even the implied warranty of
;; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
;; GNU General Public License for more details.

;; You should have received a copy of the GNU General Public License
;; along with GNU Emacs; see the file COPYING.  If not, write to
;; the Free Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.

;; There is still major problems with the mudlle-mode.
;; Too much C cruft is still here.

(defvar mudlle-mode-abbrev-table nil
  "Abbrev table in use in Mudlle-mode buffers.")
(define-abbrev-table 'mudlle-mode-abbrev-table ())

(defvar mudlle-mode-map ()
  "Keymap used in mudlle mode.")
(if mudlle-mode-map
    ()
  (setq mudlle-mode-map (make-sparse-keymap))
  (define-key mudlle-mode-map "[" 'electric-mudlle-bracket)
  (define-key mudlle-mode-map "]" 'electric-mudlle-bracket)
  (define-key mudlle-mode-map ";" 'electric-mudlle-semi)
  (define-key mudlle-mode-map "\e\C-h" 'mark-mudlle-function)
  (define-key mudlle-mode-map "\e\C-q" 'indent-mudlle-exp)
  (define-key mudlle-mode-map "\177" 'backward-delete-char-untabify)
  (define-key mudlle-mode-map "\t" 'mudlle-indent-command))

(defvar mudlle-mode-syntax-table nil
  "Syntax table in use in Mudlle-mode buffers.")

(if mudlle-mode-syntax-table
    ()
  (setq mudlle-mode-syntax-table (make-syntax-table))
  (modify-syntax-entry ?\\ "\\" mudlle-mode-syntax-table)
  (modify-syntax-entry ?/ ". 12" mudlle-mode-syntax-table)
  (modify-syntax-entry ?* "." mudlle-mode-syntax-table)
  (modify-syntax-entry ?+ "." mudlle-mode-syntax-table)
  (modify-syntax-entry ?- "." mudlle-mode-syntax-table)
  (modify-syntax-entry ?= "." mudlle-mode-syntax-table)
  (modify-syntax-entry ?% "." mudlle-mode-syntax-table)
  (modify-syntax-entry ?< "." mudlle-mode-syntax-table)
  (modify-syntax-entry ?> "." mudlle-mode-syntax-table)
  (modify-syntax-entry ?& "." mudlle-mode-syntax-table)
  (modify-syntax-entry ?| "." mudlle-mode-syntax-table)
  (modify-syntax-entry ?\n ">" mudlle-mode-syntax-table)
  (modify-syntax-entry ?' "'" mudlle-mode-syntax-table))

(defvar mudlle-indent-level 4
  "*Indentation of mudlle statements with respect to containing block.")
(defvar mudlle-bracket-imaginary-offset 0
  "*Imagined indentation of a mudlle open bracket that actually follows a
statement.")
(defvar mudlle-bracket-offset 0
  "*Extra indentation for brackets, compared with other text in same context.")
(defvar mudlle-argdecl-indent 4
  "*Indentation level of declarations of mudlle function arguments.")
(defvar mudlle-label-offset -2
  "*Offset of mudlle label lines and case statements relative to usual
indentation.")
(defvar mudlle-continued-statement-offset 4
  "*Extra indent for lines not starting new statements.")
(defvar mudlle-continued-bracket-offset 0
  "*Extra indent for substatements that start with open-brackets.
This is in addition to mudlle-continued-statement-offset.")

(defvar mudlle-auto-newline nil
  "*Non-nil means automatically newline before and after brackets,
and after colons and semicolons, inserted in mudlle code.")

(defvar mudlle-tab-always-indent nil
  "*Non-nil means TAB in mudlle mode should always reindent the current line,
regardless of where in the line point is when the TAB command is used.")

(defun mudlle-mode ()
  "Major mode for editing mudlle code.
Expression and list commands understand all mudlle brackets.
Tab indents for mudlle code.
Comments are delimited with // and end of line.
Paragraphs are separated by blank lines only.
Delete converts tabs to spaces as it moves back.
\\{mudlle-mode-map}
Variables controlling indentation style:
 mudlle-tab-always-indent
    Non-nil means TAB in mudlle mode should always reindent the current line,
    regardless of where in the line point is when the TAB command is used.
 mudlle-auto-newline
    Non-nil means automatically newline before and after brackets,
    and after colons and semicolons, inserted in mudlle code.
 mudlle-indent-level
    Indentation of mudlle statements within surrounding block.
    The surrounding block's indentation is the indentation
    of the line on which the open-bracket appears.
 mudlle-continued-statement-offset
    Extra indentation given to a substatement, such as the
    then-clause of an if or body of a while.
 mudlle-continued-bracket-offset
    Extra indentation given to a bracket that starts a substatement.
    This is in addition to mudlle-continued-statement-offset.
 mudlle-bracket-offset
    Extra indentation for line if it starts with an open bracket.
 mudlle-bracket-imaginary-offset
    An open bracket following other text is treated as if it were
    this far to the right of the start of its line.
 mudlle-argdecl-indent
    Indentation level of declarations of mudlle function arguments.
 mudlle-label-offset
    Extra indentation for line that is a label, or case or default.

Turning on mudlle mode calls the value of the variable mudlle-mode-hook with
no args, if that value is non-nil."
  (interactive)
  (kill-all-local-variables)
  (use-local-map mudlle-mode-map)
  (setq major-mode 'mudlle-mode)
  (setq mode-name "mudlle")
  (setq local-abbrev-table mudlle-mode-abbrev-table)
  (set-syntax-table mudlle-mode-syntax-table)
  (make-local-variable 'paragraph-start)
  (setq paragraph-start (concat "^$\\|" page-delimiter))
  (make-local-variable 'paragraph-separate)
  (setq paragraph-separate paragraph-start)
  (make-local-variable 'paragraph-ignore-fill-prefix)
  (setq paragraph-ignore-fill-prefix t)
  (make-local-variable 'indent-line-function)
  (setq indent-line-function 'mudlle-indent-line)
  (make-local-variable 'require-final-newline)
  (setq require-final-newline t)
  (make-local-variable 'comment-start)
  (setq comment-start "// ")
  (make-local-variable 'comment-end)
  (setq comment-end "")
  (make-local-variable 'comment-column)
  (setq comment-column 32)
  (make-local-variable 'comment-start-skip)
  (setq comment-start-skip "// *")
  (make-local-variable 'comment-indent-hook)
  (setq comment-indent-hook 'mudlle-comment-indent)
  (make-local-variable 'parse-sexp-ignore-comments)
  (setq parse-sexp-ignore-comments nil)
  (run-hooks 'mudlle-mode-hook))

;; This is used by indent-for-comment
;; to decide how much to indent a comment in mudlle code
;; based on its context.
(defun mudlle-comment-indent ()
  (if (looking-at "^//")
      0				;Existing comment at bol stays there.
    (save-excursion
      (skip-chars-backward " \t")
      (max (1+ (current-column))	;Else indent at comment column
	   comment-column))))	; except leave at least one space.

(defun electric-mudlle-bracket (arg)
  "Insert character and correct line's indentation."
  (interactive "P")
  (let (insertpos)
    (if (and (not arg)
	     (eolp)
	     (or (save-excursion
		   (skip-chars-backward " \t")
		   (bolp))
		 (if mudlle-auto-newline
		     (progn (mudlle-indent-line) (newline) t) nil)))
	(progn
	  (insert last-command-char)
	  (mudlle-indent-line)
	  (if mudlle-auto-newline
	      (progn
		(newline)
		;; (newline) may have done auto-fill
		(setq insertpos (- (point) 2))
		(mudlle-indent-line)))
	  (save-excursion
	    (if insertpos (goto-char (1+ insertpos)))
	    (delete-char -1))))
    (if insertpos
	(save-excursion
	  (goto-char insertpos)
	  (self-insert-command (prefix-numeric-value arg)))
      (self-insert-command (prefix-numeric-value arg)))))

(defun electric-mudlle-semi (arg)
  "Insert character and correct line's indentation."
  (interactive "P")
  (if mudlle-auto-newline
      (electric-mudlle-terminator arg)
    (self-insert-command (prefix-numeric-value arg))))

(defun electric-mudlle-terminator (arg)
  "Insert character and correct line's indentation."
  (interactive "P")
  (let (insertpos (end (point)))
    (if (and (not arg) (eolp)
	     (not (save-excursion
		    (beginning-of-line)
		    (skip-chars-forward " \t")
		    (progn
		      (beginning-of-defun)
		      (let ((pps (parse-partial-sexp (point) end)))
			(or (nth 3 pps) (nth 4 pps) (nth 5 pps)))))))
	(progn
	  (insert last-command-char)
	  (mudlle-indent-line)
	  (and mudlle-auto-newline
	       (not (mudlle-inside-parens-p))
	       (progn
		 (newline)
		 (setq insertpos (- (point) 2))
		 (mudlle-indent-line)))
	  (save-excursion
	    (if insertpos (goto-char (1+ insertpos)))
	    (delete-char -1))))
    (if insertpos
	(save-excursion
	  (goto-char insertpos)
	  (self-insert-command (prefix-numeric-value arg)))
      (self-insert-command (prefix-numeric-value arg)))))

(defun mudlle-inside-parens-p ()
  (condition-case ()
      (save-excursion
	(save-restriction
	  (narrow-to-region (point)
			    (progn (beginning-of-defun) (point)))
	  (goto-char (point-max))
	  (= (char-after (or (scan-lists (point) -1 1) (point-min))) ?\()))
    (error nil)))

(defun mudlle-indent-command (&optional whole-exp)
  "Indent current line as mudlle code, or in some cases insert a tab character.
If mudlle-tab-always-indent is non-nil (the default), always indent current
line.
Otherwise, indent the current line only if point is at the left margin
or in the line's indentation; otherwise insert a tab.

A numeric argument, regardless of its value,
means indent rigidly all the lines of the expression starting after point
so that this line becomes properly indented.
The relative indentation among the lines of the expression are preserved."
  (interactive "P")
  (if whole-exp
      ;; If arg, always indent this line as mudlle
      ;; and shift remaining lines of expression the same amount.
      (let ((shift-amt (mudlle-indent-line))
	    beg end)
	(save-excursion
	  (if mudlle-tab-always-indent
	      (beginning-of-line))
	  (setq beg (point))
	  (forward-sexp 1)
	  (setq end (point))
	  (goto-char beg)
	  (forward-line 1)
	  (setq beg (point)))
	(if (> end beg)
	    (indent-code-rigidly beg end shift-amt "#")))
    (if (and (not mudlle-tab-always-indent)
	     (save-excursion
	       (skip-chars-backward " \t")
	       (not (bolp))))
	(insert-tab)
      (mudlle-indent-line))))

(defun mudlle-indent-line ()
  "Indent current line as mudlle code.
Return the amount the indentation changed by."
  (let ((indent (calculate-mudlle-indent nil))
	beg shift-amt
	(case-fold-search nil)
	(pos (- (point-max) (point))))
    (beginning-of-line)
    (setq beg (point))
    (cond ((eq indent nil)
	   (setq indent (current-indentation)))
	  ((looking-at "[ \t]*#")
	   (setq indent 0))
	  (t
	   (skip-chars-forward " \t")
	   (if (listp indent) (setq indent (car indent)))
	   (cond ((and (looking-at "else\\b")
		       (not (looking-at "else\\s_")))
		  (setq indent (save-excursion
				 (mudlle-backward-to-start-of-if)
				 (current-indentation))))
		 ((= (following-char) ?])
		  (setq indent (- indent mudlle-indent-level)))
		 ((= (following-char) ?[)
		  (setq indent (+ indent mudlle-bracket-offset))))))
    (skip-chars-forward " \t")
    (setq shift-amt (- indent (current-column)))
    (if (zerop shift-amt)
	(if (> (- (point-max) pos) (point))
	    (goto-char (- (point-max) pos)))
      (delete-region beg (point))
      (indent-to indent)
      ;; If initial point was within line's indentation,
      ;; position after the indentation.  Else stay at same point in text.
      (if (> (- (point-max) pos) (point))
	  (goto-char (- (point-max) pos))))
    shift-amt))

(defun calculate-mudlle-indent (&optional parse-start)
  "Return appropriate indentation for current line as mudlle code.
In usual case returns an integer: the column to indent to.
Returns nil if line starts inside a string, t if in a comment."
  (save-excursion
    (beginning-of-line)
    (let ((indent-point (point))
	  (case-fold-search nil)
	  state
	  containing-sexp)
      (if parse-start
	  (goto-char parse-start)
	(beginning-of-defun))
      (while (< (point) indent-point)
	(setq parse-start (point))
	(setq state (parse-partial-sexp (point) indent-point 0))
	(setq containing-sexp (car (cdr state))))
      (cond ((or (nth 3 state) (nth 4 state))
	     ;; return nil or t if should not change this line
	     (nth 4 state))
	    ((null containing-sexp)
	     ;; Line is at top level.  May be data or function definition,
	     ;; or may be function argument declaration.
	     ;; Indent like the previous top level line
	     ;; unless that ends in a closeparen without semicolon,
	     ;; in which case this line is the first argument decl.
	     (goto-char indent-point)
	     (skip-chars-forward " \t")
	     (if (= (following-char) ?[)
		 0   ; Unless it starts a function body
	       (mudlle-backward-to-noncomment (or parse-start (point-min)))
	       ;; Look at previous line that's at column 0
	       ;; to determine whether we are in top-level decls
	       ;; or function's arg decls.  Set basic-indent accordinglu.
	       (let ((basic-indent
		      (save-excursion
			(re-search-backward "^[^ \^L\t\n#]" nil 'move)
			(if (and (looking-at "\\sw\\|\\s_")
				 (looking-at "[^\"\n=(]*(")
				 (progn
				   (goto-char (1- (match-end 0)))
				   (forward-sexp 1)
				   (and (< (point) indent-point)
					(not (memq (following-char)
						   '(?\, ?\;))))))
			    mudlle-argdecl-indent 0))))
		 ;; Now add a little if this is a continuation line.
		 (+ basic-indent (if (or (bobp)
					 (memq (preceding-char)
					       '(?\) ?\; ?\])))
				     0 mudlle-continued-statement-offset)))))
	    ((/= (char-after containing-sexp) ?[)
	     ;; line is expression, not statement:
	     ;; indent to just after the surrounding open.
	     (goto-char (1+ containing-sexp))
	     (current-column))
	    (t
	     ;; Statement level.  Is it a continuation or a new statement?
	     ;; Find previous non-comment character.
	     (goto-char indent-point)
	     (mudlle-backward-to-noncomment containing-sexp)
	     ;; Back up over label lines, since they don't
	     ;; affect whether our line is a continuation.
	     (while (or (eq (preceding-char) ?\,)
			(and (eq (preceding-char) ?:)
			     (or (eq (char-after (- (point) 2)) ?\')
				 (memq (char-syntax (char-after (- (point) 2)))
				       '(?w ?_)))))
	       (if (eq (preceding-char) ?\,)
		   (mudlle-backward-to-start-of-continued-exp containing-sexp))
	       (beginning-of-line)
	       (mudlle-backward-to-noncomment containing-sexp))
	     ;; Now we get the answer.
	     (if (not (memq (preceding-char) '(nil ?\, ?\; ?\] ?\[)))
		 ;; This line is continuation of preceding line's statement;
		 ;; indent  mudlle-continued-statement-offset  more than the
		 ;; previous line of the statement.
		 (progn
		   (mudlle-backward-to-start-of-continued-exp containing-sexp)
		   (+ mudlle-continued-statement-offset (current-column)
		      (if (save-excursion (goto-char indent-point)
					  (skip-chars-forward " \t")
					  (eq (following-char) ?[))
			  mudlle-continued-bracket-offset 0)))
	       ;; This line starts a new statement.
	       ;; Position following last unclosed open.
	       (goto-char containing-sexp)
	       ;; Is line first statement after an open-bracket?
	       (or
		 ;; If no, find that first statement and indent like it.
		 (save-excursion
		   (forward-char 1)
		   (let ((colon-line-end 0))
		     (while
			 (progn (skip-chars-forward " \t\n")
				(looking-at
				 "#\\|//\\|case[ \t\n].*:\\|[a-zA-Z0-9_$]*:"))
		       ;; Skip over comments and labels following openbracket.
		       (cond ((= (following-char) ?\#)
			      (forward-line 1))
			     ((= (following-char) ?\/)
			      (forward-char 2)
			      (search-forward "*/" nil 'move))
			     ;; case or label:
			     (t
			      (save-excursion (end-of-line)
					      (setq colon-line-end (point)))
			      (search-forward ":"))))
		     ;; The first following code counts
		     ;; if it is before the line we want to indent.
		     (and (< (point) indent-point)
			  (- 
			   (if (> colon-line-end (point))
			       (- (current-indentation) mudlle-label-offset)
			     (current-column))
			   (if (= (following-char) ?\[)
			       mudlle-bracket-offset 0)))))
		 ;; If no previous statement,
		 ;; indent it relative to line bracket is on.
		 ;; For open bracket in column zero, don't let statement
		 ;; start there too.  If mudlle-indent-level is zero,
		 ;; use mudlle-bracket-offset
		 ;; + mudlle-continued-statement-offset instead.
		 ;; For open-brackets not the first thing in a line,
		 ;; add in mudlle-bracket-imaginary-offset.
		 (+ (if (and (bolp) (zerop mudlle-indent-level))
			(+ mudlle-bracket-offset
			   mudlle-continued-statement-offset)
		      mudlle-indent-level)
		    ;; Move back over whitespace before the openbracket.
		    ;; If openbracket is not first nonwhite thing on the line,
		    ;; add the mudlle-bracket-imaginary-offset.
		    (progn (skip-chars-backward " \t")
			   (if (bolp) 0 mudlle-bracket-imaginary-offset))
		    ;; If the openbracket is preceded by a parenthesized exp,
		    ;; move to the beginning of that;
		    ;; possibly a different line
		    (progn
		      (if (eq (preceding-char) ?\))
			  (forward-sexp -1))
		      ;; Get initial indentation of the line we are on.
		      (current-indentation))))))))))

(defun mudlle-backward-to-noncomment (lim)
  (let (opoint stop)
    (while (not stop)
      (skip-chars-backward " \t\n\f" lim)
      (setq opoint (point))
      (if (and (>= (point) (+ 2 lim))
	       (save-excursion
		 (forward-char -2)
		 (looking-at "\\*/")))
	  (search-backward "//" lim 'move)
	(setq stop (or (<= (point) lim)
		       (save-excursion
			 (beginning-of-line)
			 (skip-chars-forward " \t")
			 (not (looking-at "#")))))
	(or stop (beginning-of-line))))))

;; new version of the above
(defun mudlle-backward-to-noncomment-new (lim)
  (let (stop)
    (while (not stop)
      ;; find comment on this line
      (beginning-of-line)
      (search-forward "//" (save-excursion (end-of-line) (point)) t)
      (setq stop (or (<= (point) lim)
		     (save-excursion
			 (beginning-of-line)
			 (skip-chars-forward " \t")
			 (not (looking-at "#"))))))))


(defun mudlle-backward-to-start-of-continued-exp (lim)
  (if (= (preceding-char) ?\))
      (forward-sexp -1))
  (beginning-of-line)
  (if (<= (point) lim)
      (goto-char (1+ lim)))
  (skip-chars-forward " \t"))

(defun mudlle-backward-to-start-of-if (&optional limit)
  "Move to the start of the last ``unbalanced'' if."
  (or limit (setq limit (save-excursion (beginning-of-defun) (point))))
  (let ((if-level 1)
	(case-fold-search nil))
    (while (not (zerop if-level))
      (backward-sexp 1)
      (cond ((looking-at "else\\b")
	     (setq if-level (1+ if-level)))
	    ((looking-at "if\\b")
	     (setq if-level (1- if-level)))
	    ((< (point) limit)
	     (setq if-level 0)
	     (goto-char limit))))))


(defun mark-mudlle-function ()
  "Put mark at end of mudlle function, point at beginning."
  (interactive)
  (push-mark (point))
  (end-of-defun)
  (push-mark (point))
  (beginning-of-defun)
  (backward-paragraph))

(defun indent-mudlle-exp ()
  "Indent each line of the mudlle grouping following point."
  (interactive)
  (let ((indent-stack (list nil))
	(contain-stack (list (point)))
	(case-fold-search nil)
	restart outer-loop-done inner-loop-done state ostate
	this-indent last-sexp
	at-else at-bracket
	(opoint (point))
	(next-depth 0))
    (save-excursion
      (forward-sexp 1))
    (save-excursion
      (setq outer-loop-done nil)
      (while (and (not (eobp)) (not outer-loop-done))
	(setq last-depth next-depth)
	;; Compute how depth changes over this line
	;; plus enough other lines to get to one that
	;; does not end inside a comment or string.
	;; Meanwhile, do appropriate indentation on comment lines.
	(setq inner-loop-done nil)
	(while (and (not inner-loop-done)
		    (not (and (eobp) (setq outer-loop-done t))))
	  (setq ostate state)
	  (setq state (parse-partial-sexp (point) (progn (end-of-line) (point))
					  nil nil state))
	  (setq next-depth (car state))
	  (if (and (car (cdr (cdr state)))
		   (>= (car (cdr (cdr state))) 0))
	      (setq last-sexp (car (cdr (cdr state)))))
	  (if (or (nth 4 ostate))
	      (mudlle-indent-line))
	  (if (or (nth 3 state))
	      (forward-line 1)
	    (setq inner-loop-done t)))
	(if (<= next-depth 0)
	    (setq outer-loop-done t))
	(if outer-loop-done
	    nil
	  ;; If this line had ..))) (((.. in it, pop out of the levels
	  ;; that ended anywhere in this line, even if the final depth
	  ;; doesn't indicate that they ended.
	  (while (> last-depth (nth 6 state))
	    (setq indent-stack (cdr indent-stack)
		  contain-stack (cdr contain-stack)
		  last-depth (1- last-depth)))
	  (if (/= last-depth next-depth)
	      (setq last-sexp nil))
	  ;; Add levels for any parens that were started in this line.
	  (while (< last-depth next-depth)
	    (setq indent-stack (cons nil indent-stack)
		  contain-stack (cons nil contain-stack)
		  last-depth (1+ last-depth)))
	  (if (null (car contain-stack))
	      (setcar contain-stack (or (car (cdr state))
					(save-excursion (forward-sexp -1)
							(point)))))
	  (forward-line 1)
	  (skip-chars-forward " \t")
	  (if (eolp)
	      nil
	    (if (and (car indent-stack)
		     (>= (car indent-stack) 0))
		;; Line is on an existing nesting level.
		;; Lines inside parens are handled specially.
		(if (/= (char-after (car contain-stack)) ?[)
		    (setq this-indent (car indent-stack))
		  ;; Line is at statement level.
		  ;; Is it a new statement?  Is it an else?
		  ;; Find last non-comment character before this line
		  (save-excursion
		    (setq at-else (looking-at "else\\W"))
		    (setq at-bracket (= (following-char) ?[))
		    (mudlle-backward-to-noncomment opoint)
		    (if (not (memq (preceding-char) '(nil ?\, ?\; ?] ?: ?[)))
			;; Preceding line did not end in comma or semi;
			;; indent this line  mudlle-continued-statement-offset
			;; more than previous.
			(progn
			  (mudlle-backward-to-start-of-continued-exp
			   (car contain-stack))
			  (setq this-indent
				(+ mudlle-continued-statement-offset
				   (current-column)
				   (if at-bracket
				       mudlle-continued-bracket-offset
				     0))))
		      ;; Preceding line ended in comma or semi;
		      ;; use the standard indent for this level.
		      (if at-else
			  (progn (mudlle-backward-to-start-of-if opoint)
				 (setq this-indent (current-indentation)))
			(setq this-indent (car indent-stack))))))
	      ;; Just started a new nesting level.
	      ;; Compute the standard indent for this level.
	      (let ((val (calculate-mudlle-indent
			   (if (car indent-stack)
			       (- (car indent-stack))))))
		(setcar indent-stack
			(setq this-indent val))))
	    ;; Adjust line indentation according to its contents
	    (if (or (looking-at "case[ \t]")
		    (and (looking-at "[A-Za-z]")
			 (save-excursion
			   (forward-sexp 1)
			   (looking-at ":"))))
		(setq this-indent (max 1 (+ this-indent mudlle-label-offset))))
	    (if (= (following-char) ?])
		(setq this-indent (- this-indent mudlle-indent-level)))
	    (if (= (following-char) ?[)
		(setq this-indent (+ this-indent mudlle-bracket-offset)))
	    ;; Put chosen indentation into effect.
	    (or (= (current-column) this-indent)
		(= (following-char) ?\#)
		(progn
		  (delete-region (point) (progn (beginning-of-line) (point)))
		  (indent-to this-indent)))
	    ;; Indent any comment following the text.
	    (or (looking-at comment-start-skip)
		(if (re-search-forward
		     comment-start-skip
		     (save-excursion (end-of-line) (point)) t)
		    (progn (indent-for-comment) (beginning-of-line)))))))))
; (message "Indenting mudlle expression...done")
  )

